@@ -1,5 +1,5 @@
 use crate::{
-    error::PRes,
+    error::PERes,
     io::{
         read_u64, write_u64, InfallibleRead, InfallibleReadFormat, InfallibleWrite, InfallibleWriteFormat, ReadFormat,
         WriteFormat,
@@ -12,57 +12,60 @@ use std::{
     sync::{Arc, Mutex},
 };
 pub const PAGE_METADATA_SIZE: u32 = 3;
-
+const SIZE_OFFSET: usize = 0;
+const FLAGS_OFFSET: usize = SIZE_OFFSET + 1;
+const NEXT_OFFSET: usize = FLAGS_OFFSET + 1;
+const PREV_OFFSET: usize = NEXT_OFFSET + 8;
 pub trait UpdateList {
-    fn update(&mut self, exp: u8, page: u64) -> PRes<u64>;
-    fn remove(&mut self, exp: u8, page: u64, next: u64) -> PRes<()>;
+    fn update(&mut self, exp: u8, page: u64) -> PERes<u64>;
+    fn remove(&mut self, exp: u8, page: u64, next: u64) -> PERes<()>;
 }
 
 pub trait Device: Sync + Send {
-    fn load_page(&self, page: u64) -> PRes<ReadPage>;
+    fn load_page(&self, page: u64) -> PERes<ReadPage>;
 
-    fn load_page_if_exists(&self, page: u64) -> PRes<Option<ReadPage>>;
+    fn load_page_if_exists(&self, page: u64) -> PERes<Option<ReadPage>>;
 
     /// Load a page avoiding to skip base page metadata, used for root page or metadata
     /// manipulation.
     ///
-    fn load_page_raw(&self, page: u64, size_exp: u8) -> PRes<Page>;
+    fn load_page_raw(&self, page: u64, size_exp: u8) -> PERes<Page>;
 
-    fn flush_page(&self, page: &Page) -> PRes<()>;
+    fn flush_page(&self, page: &Page) -> PERes<()>;
 
     /// Create a page without setting metadata, used by root page
-    fn create_page_raw(&self, exp: u8) -> PRes<u64>;
+    fn create_page_raw(&self, exp: u8) -> PERes<u64>;
 
-    fn create_page(&self, exp: u8) -> PRes<Page>;
+    fn create_page(&self, exp: u8) -> PERes<Page>;
 
-    fn mark_allocated(&self, page: u64) -> PRes<u64>;
+    fn mark_allocated(&self, page: u64) -> PERes<u64>;
 
-    fn sync(&self) -> PRes<()>;
+    fn sync(&self) -> PERes<()>;
 
-    fn trim_or_free_page(&self, page: u64, update_list: &mut dyn UpdateList) -> PRes<()>;
+    fn trim_or_free_page(&self, page: u64, update_list: &mut dyn UpdateList) -> PERes<()>;
 
-    fn trim_end_pages(&self, update_list: &mut dyn UpdateList) -> PRes<()>;
+    fn trim_end_pages(&self, update_list: &mut dyn UpdateList) -> PERes<()>;
 }
 
 trait SizeTool {
-    fn len(&self) -> PRes<u64>;
-    fn set_len(&mut self, len: u64) -> PRes<()>;
+    fn len(&self) -> PERes<u64>;
+    fn set_len(&mut self, len: u64) -> PERes<()>;
 }
 
 impl SizeTool for File {
-    fn len(&self) -> PRes<u64> {
+    fn len(&self) -> PERes<u64> {
         Ok(self.metadata()?.len())
     }
-    fn set_len(&mut self, len: u64) -> PRes<()> {
+    fn set_len(&mut self, len: u64) -> PERes<()> {
         Ok(File::set_len(self, len)?)
     }
 }
 
 impl SizeTool for Cursor<Vec<u8>> {
-    fn len(&self) -> PRes<u64> {
+    fn len(&self) -> PERes<u64> {
         Ok(self.get_ref().len() as u64)
     }
-    fn set_len(&mut self, len: u64) -> PRes<()> {
+    fn set_len(&mut self, len: u64) -> PERes<()> {
         self.get_mut().resize(len as usize, 0);
         Ok(())
     }
@@ -124,7 +127,7 @@ impl PageOps for ReadPage {
     fn clone_write(&self) -> Page {
         Page::new(self.buff.as_ref().clone(), 2, self.index, self.exp)
     }
-    fn is_free(&self) -> PRes<bool> {
+    fn is_free(&self) -> PERes<bool> {
         Ok((self.buff[1] & 0b1000_0000) != 0)
     }
     fn cursor_pos(&self) -> usize {
@@ -151,7 +154,7 @@ pub trait PageOps {
     fn get_size_exp(&self) -> u8;
     fn clone_read(&self) -> ReadPage;
     fn clone_write(&self) -> Page;
-    fn is_free(&self) -> PRes<bool>;
+    fn is_free(&self) -> PERes<bool>;
     fn cursor_pos(&self) -> usize;
 }
 
@@ -237,7 +240,7 @@ impl PageOps for Page {
     fn clone_write(&self) -> Page {
         self.clone_resetted()
     }
-    fn is_free(&self) -> PRes<bool> {
+    fn is_free(&self) -> PERes<bool> {
         Ok(is_free(self.buff[1]))
     }
     fn cursor_pos(&self) -> usize {
@@ -250,21 +253,21 @@ impl PageOps for Page {
 }
 
 impl Page {
-    pub fn set_free(&mut self, free: bool) -> PRes<()> {
+    pub fn set_free(&mut self, free: bool) -> PERes<()> {
         self.buff[1] = free_flag_set(self.buff[1], free);
         Ok(())
     }
 
     pub fn set_next_free(&mut self, next: u64) {
         let pre = self.pos;
-        self.pos = 2;
+        self.pos = NEXT_OFFSET;
         InfallibleWriteFormat::write_u64(self, next);
         self.pos = pre;
     }
 
     pub fn get_next_free(&mut self) -> u64 {
         let pre = self.pos;
-        self.pos = 2;
+        self.pos = NEXT_OFFSET;
         let val = InfallibleReadFormat::read_u64(self);
         self.pos = pre;
         val
@@ -272,20 +275,20 @@ impl Page {
 
     pub fn set_prev_free(&mut self, next: u64) {
         let pre = self.pos;
-        self.pos = 10;
+        self.pos = PREV_OFFSET;
         InfallibleWriteFormat::write_u64(self, next);
         self.pos = pre;
     }
 
     pub fn get_prev_free(&mut self) -> u64 {
         let pre = self.pos;
-        self.pos = 10;
+        self.pos = PREV_OFFSET;
         let val = InfallibleReadFormat::read_u64(self);
         self.pos = pre;
         val
     }
 
-    pub fn reset(&mut self) -> PRes<()> {
+    pub fn reset(&mut self) -> PERes<()> {
         self.buff = vec![0; self.buff.len()];
         self.buff[0] = self.exp;
         let end = self.buff.len() - 1;
@@ -298,7 +301,7 @@ impl Page {
     }
 }
 
-fn load_page<T: Read + Seek>(fl: &mut T, page: u64) -> PRes<ReadPage> {
+fn load_page<T: Read + Seek>(fl: &mut T, page: u64) -> PERes<ReadPage> {
     // add 2 to skip the metadata
     fl.seek(SeekFrom::Start(page))?;
     let exp = fl.read_u8()?;
@@ -309,7 +312,7 @@ fn load_page<T: Read + Seek>(fl: &mut T, page: u64) -> PRes<ReadPage> {
     Ok(ReadPage::new(Arc::new(ve), 2, page, exp))
 }
 
-fn create_page_raw<T: Write + Seek>(fl: &mut T, exp: u8) -> PRes<u64> {
+fn create_page_raw<T: Write + Seek>(fl: &mut T, exp: u8) -> PERes<u64> {
     let size: u64 = (1 << exp) as u64; //EXP - (size_exp+size_mitigator);
     let ve = vec![0u8; size as usize];
     let offset = fl.seek(SeekFrom::End(0))?;
@@ -317,7 +320,7 @@ fn create_page_raw<T: Write + Seek>(fl: &mut T, exp: u8) -> PRes<u64> {
     Ok(offset)
 }
 
-fn load_page_raw<T: Read + Seek>(fl: &mut T, page: u64, size_exp: u8) -> PRes<Page> {
+fn load_page_raw<T: Read + Seek>(fl: &mut T, page: u64, size_exp: u8) -> PERes<Page> {
     let mut ve;
     let size;
     {
@@ -329,13 +332,13 @@ fn load_page_raw<T: Read + Seek>(fl: &mut T, page: u64, size_exp: u8) -> PRes<Pa
     Ok(Page::new(ve, 0, page, size_exp))
 }
 
-fn flush_page<T: Write + Seek>(fl: &mut T, page: &Page) -> PRes<()> {
+fn flush_page<T: Write + Seek>(fl: &mut T, page: &Page) -> PERes<()> {
     fl.seek(SeekFrom::Start(page.index))?;
     fl.write_all(&page.buff)?;
     Ok(())
 }
 
-fn create_page<T: Write + Seek>(fl: &mut T, exp: u8) -> PRes<Page> {
+fn create_page<T: Write + Seek>(fl: &mut T, exp: u8) -> PERes<Page> {
     let size: u64 = (1 << exp) as u64; //EXP - (size_exp+size_mitigator);
     let mut ve = vec![0u8; size as usize];
     ve[0] = exp;
@@ -345,19 +348,19 @@ fn create_page<T: Write + Seek>(fl: &mut T, exp: u8) -> PRes<Page> {
     Ok(Page::new(ve, 2, offset, exp))
 }
 
-fn mark_allocated<T: Write + Read + Seek>(fl: &mut T, page: u64) -> PRes<u64> {
-    fl.seek(SeekFrom::Start(page + 2))?;
-    // Free pages are a linked list reading the next page and return
-    let next = fl.read_u64()?;
-    fl.seek(SeekFrom::Start(page + 1))?;
+fn mark_allocated<T: Write + Read + Seek>(fl: &mut T, page: u64) -> PERes<u64> {
+    fl.seek(SeekFrom::Start(page + FLAGS_OFFSET as u64))?;
     let mut moderator = fl.read_u8()?;
+    fl.seek(SeekFrom::Start(page + NEXT_OFFSET as u64))?;
+    // Free pages are a linked list reading the next page to return
+    let next = fl.read_u64()?;
     moderator = free_flag_set(moderator, false);
     fl.seek(SeekFrom::Start(page + 1))?;
     fl.write_u8(moderator)?;
     fl.write_u64(0)?; // next
     fl.write_u64(0)?; // previous
     if next != 0 {
-        fl.seek(SeekFrom::Start(next + 10))?;
+        fl.seek(SeekFrom::Start(next + PREV_OFFSET as u64))?;
         fl.write_u64(0)?;
     }
     Ok(next)
@@ -367,7 +370,7 @@ fn trim_or_free_page<T: Write + Read + Seek + SizeTool>(
     fl: &mut T,
     page: u64,
     update_list: &mut dyn UpdateList,
-) -> PRes<()> {
+) -> PERes<()> {
     fl.seek(SeekFrom::Start(page))?;
     let exp = fl.read_u8()?;
     let mut mitigator = fl.read_u8()?;
@@ -377,29 +380,27 @@ fn trim_or_free_page<T: Write + Read + Seek + SizeTool>(
         fl.set_len(page)?;
         while check_and_trim(fl, update_list)? {}
     } else {
-        let old = update_list.update(exp, page)?;
+        let next = update_list.update(exp, page)?;
         fl.seek(SeekFrom::Start(page + 1))?;
         mitigator = free_flag_set(mitigator, true);
         let mut data: [u8; 9] = [0; 9];
         data[0] = mitigator;
-        write_u64(&mut data[1..9], old);
+        write_u64(&mut data[1..9], next);
         fl.seek(SeekFrom::Start(page + 1))?;
         fl.write_all(&data)?;
-        if exp >= 5 {
-            fl.write_all(&[0u8; 16])?;
-        }
+        fl.write_all(&[0u8; 8])?;
 
-        if old != 0 {
+        if next != 0 {
             let mut data = [0; 8];
             write_u64(&mut data, page);
-            fl.seek(SeekFrom::Start(old + 10))?;
+            fl.seek(SeekFrom::Start(next + PREV_OFFSET as u64))?;
             fl.write_all(&data)?;
         }
     }
     Ok(())
 }
 
-fn check_and_trim<T: Write + Read + Seek + SizeTool>(fl: &mut T, update_list: &mut dyn UpdateList) -> PRes<bool> {
+fn check_and_trim<T: Write + Read + Seek + SizeTool>(fl: &mut T, update_list: &mut dyn UpdateList) -> PERes<bool> {
     let len = fl.seek(SeekFrom::End(0))?;
     if len == 0 {
         return Ok(false);
@@ -410,7 +411,7 @@ fn check_and_trim<T: Write + Read + Seek + SizeTool>(fl: &mut T, update_list: &m
     if exp == 0 {
         return Ok(false);
     }
-    let size = (1 << exp) as u64; //EXP - (size_exp+size_mitigator);
+    let size = (1 << exp) as u64;
     let mut header = [0u8; 18];
     let page = len - size;
     if page == 0 {
@@ -419,20 +420,20 @@ fn check_and_trim<T: Write + Read + Seek + SizeTool>(fl: &mut T, update_list: &m
     fl.seek(SeekFrom::Start(page))?;
     fl.read_exact(&mut header)?;
     if is_free(header[1]) {
-        let next = read_u64(&header[2..10]);
-        let prev = read_u64(&header[10..18]);
+        let next = read_u64(&header[NEXT_OFFSET..(NEXT_OFFSET + 8)]);
+        let prev = read_u64(&header[PREV_OFFSET..(PREV_OFFSET + 8)]);
         if prev == 0 {
             update_list.remove(exp, page, next)?;
         } else {
             let mut data = [0u8; 8];
             write_u64(&mut data, next);
-            fl.seek(SeekFrom::Start(prev + 2))?;
+            fl.seek(SeekFrom::Start(prev + NEXT_OFFSET as u64))?;
             fl.write_all(&data)?;
         }
         if next != 0 {
             let mut data = [0u8; 8];
             write_u64(&mut data, prev);
-            fl.seek(SeekFrom::Start(next + 10))?;
+            fl.seek(SeekFrom::Start(next + PREV_OFFSET as u64))?;
             fl.write_all(&data)?;
         }
         fl.set_len(page)?;
@@ -450,7 +451,7 @@ pub struct DiscRef {
 
 #[cfg(unix)]
 impl DiscRef {
-    pub fn new(file: File) -> PRes<DiscRef> {
+    pub fn new(file: File) -> PERes<DiscRef> {
         let len = file.len()?;
         Ok(DiscRef {
             file,
@@ -458,14 +459,14 @@ impl DiscRef {
         })
     }
 
-    fn create_page_offset(&self, size: u64) -> PRes<u64> {
+    fn create_page_offset(&self, size: u64) -> PERes<u64> {
         let mut current = self.size.lock()?;
         let page = *current;
         *current += size;
         Ok(page)
     }
 
-    fn trim_if_possible(&self, page: u64, size: u64) -> PRes<bool> {
+    fn trim_if_possible(&self, page: u64, size: u64) -> PERes<bool> {
         let mut current = self.size.lock()?;
         if page + size == *current {
             *current = page;
@@ -476,7 +477,7 @@ impl DiscRef {
         }
     }
 
-    fn check_and_trim(&self, update_list: &mut dyn UpdateList) -> PRes<bool> {
+    fn check_and_trim(&self, update_list: &mut dyn UpdateList) -> PERes<bool> {
         use std::os::unix::prelude::FileExt;
         let mut lock = self.size.lock()?;
         let value = *lock;
@@ -496,19 +497,19 @@ impl DiscRef {
         }
         self.file.read_exact_at(&mut header, page)?;
         if is_free(header[1]) {
-            let next = read_u64(&header[2..10]);
-            let prev = read_u64(&header[10..18]);
+            let next = read_u64(&header[NEXT_OFFSET..(NEXT_OFFSET + 8)]);
+            let prev = read_u64(&header[PREV_OFFSET..(PREV_OFFSET + 8)]);
             if prev == 0 {
                 update_list.remove(exp[0], page, next)?;
             } else {
                 let mut data = [0u8; 8];
                 write_u64(&mut data, next);
-                self.file.write_all_at(&data, prev + 2)?;
+                self.file.write_all_at(&data, prev + NEXT_OFFSET as u64)?;
             }
             if next != 0 {
                 let mut data = [0u8; 8];
                 write_u64(&mut data, prev);
-                self.file.write_all_at(&data, next + 10)?;
+                self.file.write_all_at(&data, next + PREV_OFFSET as u64)?;
             }
             *lock = page;
             self.file.set_len(page)?;
@@ -521,7 +522,7 @@ impl DiscRef {
 
 #[cfg(unix)]
 impl Device for DiscRef {
-    fn load_page(&self, page: u64) -> PRes<ReadPage> {
+    fn load_page(&self, page: u64) -> PERes<ReadPage> {
         use std::os::unix::prelude::FileExt;
         let mut exp = [0u8];
         self.file.read_exact_at(&mut exp, page)?;
@@ -533,7 +534,7 @@ impl Device for DiscRef {
         Ok(ReadPage::new(Arc::new(ve), 2, page, exp[0]))
     }
 
-    fn load_page_if_exists(&self, page: u64) -> PRes<Option<ReadPage>> {
+    fn load_page_if_exists(&self, page: u64) -> PERes<Option<ReadPage>> {
         {
             if *self.size.lock()? <= page {
                 return Ok(None);
@@ -542,7 +543,7 @@ impl Device for DiscRef {
         Ok(Some(self.load_page(page)?))
     }
 
-    fn load_page_raw(&self, page: u64, size_exp: u8) -> PRes<Page> {
+    fn load_page_raw(&self, page: u64, size_exp: u8) -> PERes<Page> {
         use std::os::unix::prelude::FileExt;
         let size = (1 << size_exp) as u64; //EXP - (size_exp+size_mitigator);
         let mut ve = vec![0u8; size as usize];
@@ -550,13 +551,13 @@ impl Device for DiscRef {
         Ok(Page::new(ve, 0, page, size_exp))
     }
 
-    fn flush_page(&self, page: &Page) -> PRes<()> {
+    fn flush_page(&self, page: &Page) -> PERes<()> {
         use std::os::unix::prelude::FileExt;
         self.file.write_all_at(&page.buff, page.get_index())?;
         Ok(())
     }
 
-    fn create_page_raw(&self, exp: u8) -> PRes<u64> {
+    fn create_page_raw(&self, exp: u8) -> PERes<u64> {
         use std::os::unix::prelude::FileExt;
         let size: u64 = (1 << exp) as u64; //EXP - (size_exp+size_mitigator);
         let offset = self.create_page_offset(size)?;
@@ -565,7 +566,7 @@ impl Device for DiscRef {
         Ok(offset)
     }
 
-    fn create_page(&self, exp: u8) -> PRes<Page> {
+    fn create_page(&self, exp: u8) -> PERes<Page> {
         use std::os::unix::prelude::FileExt;
         let size: u64 = (1 << exp) as u64; //EXP - (size_exp+size_mitigator);
         let offset = self.create_page_offset(size)?;
@@ -576,7 +577,7 @@ impl Device for DiscRef {
         Ok(Page::new(ve, 2, offset, exp))
     }
 
-    fn mark_allocated(&self, page: u64) -> PRes<u64> {
+    fn mark_allocated(&self, page: u64) -> PERes<u64> {
         use std::os::unix::prelude::FileExt;
         let mut mitigator_and_next = [0u8; 9]; // flags + next
         self.file.read_exact_at(&mut mitigator_and_next, page + 1)?;
@@ -585,24 +586,23 @@ impl Device for DiscRef {
             "allocating: {} already allocated ",
             page
         );
-        // Free pages are a linked list reading the next page and return
         let mut mitigator = [0u8; 17]; //flags + next + previous
         mitigator[0] = free_flag_set(mitigator_and_next[0], false);
         self.file.write_all_at(&mitigator, page + 1)?;
         let next_page = read_u64(&mitigator_and_next[1..]);
         if next_page != 0 {
             let prev_data = [0u8; 8];
-            self.file.write_all_at(&prev_data, next_page + 10)?;
+            self.file.write_all_at(&prev_data, next_page + PREV_OFFSET as u64)?;
         }
         Ok(next_page)
     }
 
-    fn sync(&self) -> PRes<()> {
+    fn sync(&self) -> PERes<()> {
         self.file.sync_data()?;
         Ok(())
     }
 
-    fn trim_or_free_page(&self, page: u64, update_list: &mut dyn UpdateList) -> PRes<()> {
+    fn trim_or_free_page(&self, page: u64, update_list: &mut dyn UpdateList) -> PERes<()> {
         use std::os::unix::prelude::FileExt;
         let mut exp = [0u8];
         self.file.read_exact_at(&mut exp, page)?;
@@ -613,17 +613,14 @@ impl Device for DiscRef {
         if !self.trim_if_possible(page, size)? {
             let old = update_list.update(exp[0], page)?;
             mitigator[0] = free_flag_set(mitigator[0], true);
-            let mut data: [u8; 9] = [0; 9];
+            let mut data = [0u8; 17];
             data[0] = mitigator[0];
             write_u64(&mut data[1..9], old);
             self.file.write_all_at(&data, page + 1)?;
-            if exp[0] >= 5 {
-                self.file.write_all_at(&[0u8; 8], page + 10)?;
-            }
             if old != 0 {
                 let mut data = [0; 8];
                 write_u64(&mut data, page);
-                self.file.write_all_at(&data, old + 10)?;
+                self.file.write_all_at(&data, old + PREV_OFFSET as u64)?;
             }
         } else {
             self.trim_end_pages(update_list)?;
@@ -631,7 +628,7 @@ impl Device for DiscRef {
         Ok(())
     }
 
-    fn trim_end_pages(&self, update_list: &mut dyn UpdateList) -> PRes<()> {
+    fn trim_end_pages(&self, update_list: &mut dyn UpdateList) -> PERes<()> {
         while self.check_and_trim(update_list)? {}
         Ok(())
     }
@@ -650,7 +647,7 @@ pub struct DiscRef {
 
 #[cfg(not(unix))]
 impl DiscRef {
-    pub fn new(file: File) -> PRes<DiscRef> {
+    pub fn new(file: File) -> PERes<DiscRef> {
         Ok(DiscRef {
             file: Mutex::new(FileHandler {
                 file,
@@ -662,11 +659,11 @@ impl DiscRef {
 
 #[cfg(not(unix))]
 impl Device for DiscRef {
-    fn load_page(&self, page: u64) -> PRes<ReadPage> {
+    fn load_page(&self, page: u64) -> PERes<ReadPage> {
         load_page(&mut self.file.lock()?.file, page)
     }
 
-    fn load_page_if_exists(&self, page: u64) -> PRes<Option<ReadPage>> {
+    fn load_page_if_exists(&self, page: u64) -> PERes<Option<ReadPage>> {
         let mut lock = self.file.lock()?;
         if lock.file.len()? <= page {
             return Ok(None);
@@ -674,31 +671,31 @@ impl Device for DiscRef {
         Ok(Some(load_page(&mut lock.file, page)?))
     }
 
-    fn load_page_raw(&self, page: u64, size_exp: u8) -> PRes<Page> {
+    fn load_page_raw(&self, page: u64, size_exp: u8) -> PERes<Page> {
         load_page_raw(&mut self.file.lock()?.file, page, size_exp)
     }
 
-    fn flush_page(&self, page: &Page) -> PRes<()> {
+    fn flush_page(&self, page: &Page) -> PERes<()> {
         flush_page(&mut self.file.lock()?.file, page)
     }
 
-    fn create_page_raw(&self, exp: u8) -> PRes<u64> {
+    fn create_page_raw(&self, exp: u8) -> PERes<u64> {
         let lock = &mut self.file.lock()?;
         lock.metadata_changed = true;
         create_page_raw(&mut lock.file, exp)
     }
 
-    fn create_page(&self, exp: u8) -> PRes<Page> {
+    fn create_page(&self, exp: u8) -> PERes<Page> {
         let lock = &mut self.file.lock()?;
         lock.metadata_changed = true;
         create_page(&mut lock.file, exp)
     }
 
-    fn mark_allocated(&self, page: u64) -> PRes<u64> {
+    fn mark_allocated(&self, page: u64) -> PERes<u64> {
         mark_allocated(&mut self.file.lock()?.file, page)
     }
 
-    fn sync(&self) -> PRes<()> {
+    fn sync(&self) -> PERes<()> {
         let to_sync;
         let metadata_changed;
         {
@@ -715,11 +712,11 @@ impl Device for DiscRef {
         Ok(())
     }
 
-    fn trim_or_free_page(&self, page: u64, update_list: &mut dyn UpdateList) -> PRes<()> {
+    fn trim_or_free_page(&self, page: u64, update_list: &mut dyn UpdateList) -> PERes<()> {
         trim_or_free_page(&mut self.file.lock()?.file, page, update_list)
     }
 
-    fn trim_end_pages(&self, update_list: &mut dyn UpdateList) -> PRes<()> {
+    fn trim_end_pages(&self, update_list: &mut dyn UpdateList) -> PERes<()> {
         while check_and_trim(&mut self.file.lock()?.file, update_list)? {}
         Ok(())
     }
@@ -729,7 +726,7 @@ pub struct MemRef {
     data: Mutex<Cursor<Vec<u8>>>,
 }
 impl MemRef {
-    pub fn new() -> PRes<MemRef> {
+    pub fn new() -> PERes<MemRef> {
         Ok(MemRef {
             data: Mutex::new(Cursor::new(Vec::new())),
         })
@@ -737,11 +734,11 @@ impl MemRef {
 }
 
 impl Device for MemRef {
-    fn load_page(&self, page: u64) -> PRes<ReadPage> {
+    fn load_page(&self, page: u64) -> PERes<ReadPage> {
         load_page(&mut *self.data.lock()?, page)
     }
 
-    fn load_page_if_exists(&self, page: u64) -> PRes<Option<ReadPage>> {
+    fn load_page_if_exists(&self, page: u64) -> PERes<Option<ReadPage>> {
         let mut data = self.data.lock()?;
         if data.len()? <= page {
             return Ok(None);
@@ -749,34 +746,34 @@ impl Device for MemRef {
         Ok(Some(load_page(&mut *data, page)?))
     }
 
-    fn load_page_raw(&self, page: u64, size_exp: u8) -> PRes<Page> {
+    fn load_page_raw(&self, page: u64, size_exp: u8) -> PERes<Page> {
         load_page_raw(&mut *self.data.lock()?, page, size_exp)
     }
 
-    fn flush_page(&self, page: &Page) -> PRes<()> {
+    fn flush_page(&self, page: &Page) -> PERes<()> {
         flush_page(&mut *self.data.lock()?, page)
     }
 
-    fn create_page_raw(&self, exp: u8) -> PRes<u64> {
+    fn create_page_raw(&self, exp: u8) -> PERes<u64> {
         create_page_raw(&mut *self.data.lock()?, exp)
     }
 
-    fn create_page(&self, exp: u8) -> PRes<Page> {
+    fn create_page(&self, exp: u8) -> PERes<Page> {
         create_page(&mut *self.data.lock()?, exp)
     }
 
-    fn mark_allocated(&self, page: u64) -> PRes<u64> {
+    fn mark_allocated(&self, page: u64) -> PERes<u64> {
         mark_allocated(&mut *self.data.lock()?, page)
     }
 
-    fn sync(&self) -> PRes<()> {
+    fn sync(&self) -> PERes<()> {
         Ok(())
     }
 
-    fn trim_or_free_page(&self, page: u64, update_list: &mut dyn UpdateList) -> PRes<()> {
+    fn trim_or_free_page(&self, page: u64, update_list: &mut dyn UpdateList) -> PERes<()> {
         trim_or_free_page(&mut *self.data.lock()?, page, update_list)
     }
-    fn trim_end_pages(&self, update_list: &mut dyn UpdateList) -> PRes<()> {
+    fn trim_end_pages(&self, update_list: &mut dyn UpdateList) -> PERes<()> {
         while check_and_trim(&mut *self.data.lock()?, update_list)? {}
         Ok(())
     }
@@ -784,7 +781,7 @@ impl Device for MemRef {
 #[cfg(test)]
 mod tests {
     use super::{Device, DiscRef, MemRef, PageOps, UpdateList};
-    use crate::error::PRes;
+    use crate::error::PERes;
     use byteorder::{ReadBytesExt, WriteBytesExt};
     use tempfile::Builder;
 
@@ -842,10 +839,10 @@ mod tests {
 
     struct PanicCase {}
     impl UpdateList for PanicCase {
-        fn update(&mut self, _: u8, _: u64) -> PRes<u64> {
+        fn update(&mut self, _: u8, _: u64) -> PERes<u64> {
             panic!("should not put the free in the free list")
         }
-        fn remove(&mut self, _: u8, _: u64, _: u64) -> PRes<()> {
+        fn remove(&mut self, _: u8, _: u64, _: u64) -> PERes<()> {
             panic!("should not put the free in the free list")
         }
     }
@@ -873,10 +870,10 @@ mod tests {
 
     struct IgnoreCase {}
     impl UpdateList for IgnoreCase {
-        fn update(&mut self, _: u8, _: u64) -> PRes<u64> {
+        fn update(&mut self, _: u8, _: u64) -> PERes<u64> {
             Ok(0)
         }
-        fn remove(&mut self, _: u8, _: u64, _: u64) -> PRes<()> {
+        fn remove(&mut self, _: u8, _: u64, _: u64) -> PERes<()> {
             Ok(())
         }
     }
@@ -892,6 +889,106 @@ mod tests {
         assert!(load_page.unwrap().is_free().unwrap());
     }
 
+    fn create_reuse_page_device(device: &mut dyn Device) {
+        let page = device.create_page(5).unwrap().get_index();
+        let _page_after = device.create_page(5).unwrap();
+        let pg = &mut device.load_page(page).unwrap().clone_write();
+        device.flush_page(pg).unwrap();
+        device.trim_or_free_page(page, &mut IgnoreCase {}).unwrap();
+        let load_page = device.load_page(page);
+        assert!(load_page.unwrap().is_free().unwrap());
+        device.mark_allocated(page).unwrap();
+        let load_page = device.load_page(page);
+        assert!(!load_page.unwrap().is_free().unwrap());
+        device.trim_or_free_page(page, &mut IgnoreCase {}).unwrap();
+        let load_page = device.load_page(page);
+        assert!(load_page.is_ok());
+        assert!(load_page.unwrap().is_free().unwrap());
+    }
+
+    fn create_reuse_trim_page_device(device: &mut dyn Device) {
+        let page = device.create_page(5).unwrap().get_index();
+        let free_after = device.create_page(5).unwrap();
+        let pg = &mut device.load_page(page).unwrap().clone_write();
+        device.flush_page(pg).unwrap();
+        device.trim_or_free_page(page, &mut IgnoreCase {}).unwrap();
+        let load_page = device.load_page(page);
+        assert!(load_page.unwrap().is_free().unwrap());
+        device.mark_allocated(page).unwrap();
+        let load_page = device.load_page(page);
+        assert!(!load_page.unwrap().is_free().unwrap());
+        device
+            .trim_or_free_page(free_after.get_index(), &mut IgnoreCase {})
+            .unwrap();
+        device.trim_or_free_page(page, &mut IgnoreCase {}).unwrap();
+        let load_page = device.load_page(page);
+        assert!(load_page.is_err());
+    }
+
+    #[derive(Default)]
+    struct PrevCase {
+        pages: [u64; 32],
+    }
+    impl UpdateList for PrevCase {
+        fn update(&mut self, pos: u8, val: u64) -> PERes<u64> {
+            let prev = self.pages[pos as usize];
+            self.pages[pos as usize] = val;
+            Ok(prev)
+        }
+        fn remove(&mut self, pos: u8, _page: u64, next: u64) -> PERes<()> {
+            self.pages[pos as usize] = next;
+            Ok(())
+        }
+    }
+
+    fn free_tracking_device(device: &mut dyn Device) {
+        let mut update_list = PrevCase::default();
+        let _first_0 = device.create_page(5).unwrap().get_index();
+        let page = device.create_page(5).unwrap().get_index();
+        let second_page = device.create_page(5).unwrap().get_index();
+        let free_to_clean = device.create_page(5).unwrap().get_index();
+        device.trim_or_free_page(page, &mut update_list).unwrap();
+        assert_eq!(update_list.pages[5], page);
+        device.trim_or_free_page(second_page, &mut update_list).unwrap();
+        assert_eq!(update_list.pages[5], second_page);
+        let prev = device.mark_allocated(second_page).unwrap();
+        assert_eq!(prev, page);
+        let prev = device.mark_allocated(page).unwrap();
+        assert_eq!(prev, 0);
+        update_list.pages[5] = 0;
+        device.trim_or_free_page(free_to_clean, &mut update_list).unwrap();
+        device.trim_or_free_page(page, &mut update_list).unwrap();
+        assert_eq!(update_list.pages[5], page);
+        device.trim_or_free_page(second_page, &mut update_list).unwrap();
+        assert_eq!(update_list.pages[5], 0);
+        let load_page = device.load_page(page);
+        assert!(load_page.is_err());
+        let load_page = device.load_page_if_exists(page).unwrap();
+        assert!(load_page.is_none());
+        let load_page = device.load_page(second_page);
+        assert!(load_page.is_err());
+        let load_page = device.load_page_if_exists(second_page).unwrap();
+        assert!(load_page.is_none());
+    }
+
+    fn free_tracking_on_trim_device(device: &mut dyn Device) {
+        let mut update_list = PrevCase::default();
+        let _place_older_0 = device.create_page(5).unwrap().get_index();
+        let first_page = device.create_page(5).unwrap().get_index();
+        let _second_page = device.create_page(5).unwrap().get_index();
+        let third_page = device.create_page(5).unwrap().get_index();
+        let forth_page = device.create_page(5).unwrap().get_index();
+        device.trim_or_free_page(first_page, &mut update_list).unwrap();
+        assert_eq!(update_list.pages[5], first_page);
+        device.trim_or_free_page(third_page, &mut update_list).unwrap();
+        assert_eq!(update_list.pages[5], third_page);
+        device.trim_or_free_page(forth_page, &mut update_list).unwrap();
+        assert_eq!(update_list.pages[5], first_page);
+        let mut load_page = device.load_page(first_page).unwrap().clone_write();
+        assert_eq!(load_page.get_next_free(), 0);
+        assert_eq!(load_page.get_prev_free(), 0);
+    }
+
     #[test]
     fn create_load_flush_page_disc() {
         create_load_flush_page_device(&mut temp_disc_ref("disc_ref.raw"));
@@ -957,4 +1054,44 @@ mod tests {
     fn create_not_trim_not_last_memory() {
         create_not_trim_not_last_device(&mut MemRef::new().unwrap());
     }
+
+    #[test]
+    fn create_reuse_page_disc() {
+        create_reuse_page_device(&mut temp_disc_ref("disc_ref_no_trim.raw"));
+    }
+
+    #[test]
+    fn create_reuse_page_memory() {
+        create_reuse_page_device(&mut MemRef::new().unwrap());
+    }
+
+    #[test]
+    fn create_reuse_trim_page_disc() {
+        create_reuse_trim_page_device(&mut temp_disc_ref("disc_ref_no_trim.raw"));
+    }
+
+    #[test]
+    fn create_reuse_trim_page_memory() {
+        create_reuse_trim_page_device(&mut MemRef::new().unwrap());
+    }
+
+    #[test]
+    fn free_tracking_disc() {
+        free_tracking_device(&mut temp_disc_ref("disc_ref_no_trim.raw"));
+    }
+
+    #[test]
+    fn free_tracking_memory() {
+        free_tracking_device(&mut MemRef::new().unwrap());
+    }
+
+    #[test]
+    fn free_tracking_on_trim_disc() {
+        free_tracking_on_trim_device(&mut temp_disc_ref("disc_ref_trim.raw"));
+    }
+
+    #[test]
+    fn free_tracking_on_trim_memory() {
+        free_tracking_on_trim_device(&mut MemRef::new().unwrap());
+    }
 }
