@@ -185,9 +185,12 @@ impl Journal {
     }
 
     pub fn start(&self) -> PERes<JournalId> {
-        let val = self.internal_log(&Start::new(), &JournalId::new(0, 0), false)?;
+        let buffer = Self::write_buffer(&Start::new(), &JournalId::new(0, 0))?;
+        let mut jr = self.journal.lock()?;
+        self.required_space(buffer.len() as u32, &mut jr)?;
+        let val = Self::append_buffer(&mut jr, &buffer)?;
         let id = JournalId::new(val.0, val.1);
-        self.journal.lock()?.starts.push(&id);
+        jr.starts.push(&id);
         Ok(id)
     }
 
@@ -299,30 +302,35 @@ impl Journal {
         Ok(())
     }
 
-    fn internal_log(&self, entry: &dyn JournalEntry, id: &JournalId, flush: bool) -> PERes<(u64, u32)> {
+    fn write_buffer(entry: &dyn JournalEntry, id: &JournalId) -> PERes<Vec<u8>> {
         let mut buffer = Vec::<u8>::new();
         buffer.write_u8(entry.get_type());
         buffer.write_varint_u64(id.page);
         buffer.write_varint_u32(id.pos);
         entry.write(&mut buffer)?;
+        Ok(buffer)
+    }
 
-        let cur_page;
-        let cur_pos;
-        {
-            let mut jr = self.journal.lock()?;
-            self.required_space(buffer.len() as u32, &mut jr)?;
-            cur_page = jr.last_page;
-            cur_pos = jr.last_pos;
-            jr.current.seek(cur_pos);
-            jr.current.write_all(&*buffer);
-            if flush {
-                self.allocator.flush_page(jr.current.clone())?;
-            }
-            jr.last_pos += buffer.len() as u32;
-        }
+    fn append_buffer(jr: &mut MutexGuard<JournalShared>, buffer: &[u8]) -> PERes<(u64, u32)> {
+        let cur_page = jr.last_page;
+        let cur_pos = jr.last_pos;
+        jr.current.seek(cur_pos);
+        jr.current.write_all(buffer);
+        jr.last_pos += buffer.len() as u32;
         Ok((cur_page, cur_pos))
     }
 
+    fn internal_log(&self, entry: &dyn JournalEntry, id: &JournalId, flush: bool) -> PERes<()> {
+        let buffer = Self::write_buffer(entry, id)?;
+        let mut jr = self.journal.lock()?;
+        self.required_space(buffer.len() as u32, &mut jr)?;
+        Self::append_buffer(&mut jr, &buffer)?;
+        if flush {
+            self.allocator.flush_page(jr.current.clone())?;
+        }
+        Ok(())
+    }
+
     pub(crate) fn recover<T>(&self, mut found: T) -> PERes<()>
     where
         T: FnMut(&dyn JournalEntry, &JournalId),
